#ifndef FS_FILEWRITER_MT_H
#define FS_FILEWRITER_MT_H

#include "filewriter.h"
#include <list>

class FileWriterMT : public FileWriter
{
public:
	struct BufInfo
	{
		BufInfo(int inMaxSize);

		unsigned char*			mData;
		int						mLength;
		int						mSize;
		bool					mFilled;
	};

	static FileWriterMT*		GetFileWriter(const std::string& inName, long long inFileSize);
	static void					Finalize();

	virtual						~FileWriterMT();
								
	virtual bool				Open();
	virtual bool				Close();

	virtual void				GetWriteBuffer(unsigned char*& outBuf, int& outLength);
	virtual bool				ReleaseWriteBuffer(int inRemainingLength);
	
	virtual void				Finish();
	virtual bool				IsFinished()		{ return mWriteCompleted; }
	virtual void				HandleAbort();
	
	std::list<BufInfo*>&		GetBuffers()			{ return mBuffers; }
	bool						IsClosed()				{ return mClosed; }
	void						WriteCompleted()		{ mWriteCompleted = true; }

	CRITICAL_SECTION*			GetCriticalSection()	{ return &mCriticalSection; }

private :
								FileWriterMT(const std::string& inName, long long inFileSize);

	std::list<BufInfo*>			mBuffers;	
	bool						mClosed;
	bool						mWriteCompleted;
	CRITICAL_SECTION			mCriticalSection;
};

#endif